# Copyright 2009 Simon Schampijer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""HelloWorld Activity: A case study for developing an activity."""

import logging
from gettext import gettext as _
from fcntl import ioctl
import time
import os

import gtk
import gobject
import pygst
pygst.require("0.10")
import gst
import v4l2
import cairo
import hippo

from sugar.activity import activity
from sugar.activity.activity import ActivityToolbox
from sugar.graphics.toolbutton import ToolButton
from sugar.graphics.tray import TrayButton
from sugar.graphics.palette import Palette
from sugar.graphics.tray import VTray
from sugar.datastore import datastore

OLD_TOOLBAR = False

try:
    # >= 0.86 toolbars
    from sugar.graphics.toolbarbox import ToolbarButton, ToolbarBox
    from sugar.activity.widgets import ActivityToolbarButton
    from sugar.activity.widgets import StopButton
except ImportError:
    # <= 0.84 toolbars
    OLD_TOOLBAR = True

OLD_TOOLBAR = True

VD = open('/dev/video0','rw')

def generateThumbnail( pixbuf, scale, thumbw, thumbh ):
    #need to generate thumbnail version here
    thumbImg = cairo.ImageSurface(cairo.FORMAT_ARGB32, thumbw, thumbh)
    tctx = cairo.Context(thumbImg)
    img = hippo.cairo_surface_from_gdk_pixbuf(pixbuf)
    tctx.scale(scale, scale)
    tctx.set_source_surface(img, 0, 0)
    tctx.paint()
    return thumbImg

class ScalePalette(Palette):

    def __init__(self, primary_text, v4l2_control = False,
            v4l2_auto_control = False):
        Palette.__init__(self, label=primary_text)

        vbox = gtk.VBox()
        self.set_content(vbox)
        vbox.show()

        if v4l2_control:
            self._query_control = v4l2.v4l2_queryctrl(v4l2_control)
            self._control = v4l2.v4l2_control(v4l2_control)

            ioctl(VD, v4l2.VIDIOC_QUERYCTRL, self._query_control)
            ioctl(VD, v4l2.VIDIOC_G_CTRL, self._control)

            self._adjustment = gtk.Adjustment(value=self._control.value,
                    lower=self._query_control.minimum,
                    upper=self._query_control.maximum,
                    step_incr=1, page_incr=1, page_size=0)

            self._hscale = gtk.HScale(self._adjustment)
            self._hscale.set_digits(0)
            self._hscale.set_draw_value(False)
            self._hscale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
            vbox.add(self._hscale)
            self._hscale.show()

            self._adjustment_handler_id = \
                self._adjustment.connect('value_changed',
                                         self.__adjustment_changed_cb)

            if v4l2_auto_control:
                self._auto_query_control =\
                    v4l2.v4l2_queryctrl(v4l2_auto_control)
                self._auto_control = v4l2.v4l2_control(v4l2_auto_control)

                self._auto_button = gtk.CheckButton('Auto')
                self._auto_button.set_active(self._auto_control.value)
                self._auto_button.connect('toggled',
                        self.__auto_button_toggled_cb)
                vbox.add(self._auto_button)
                self._auto_button.show()

                if self._auto_control.value == True:
                    self._hscale.set_sensitive(False)

        vbox.show()

        #self.connect('popup', self.__popup_cb)

    def __auto_button_toggled_cb(self, checkbutton):
        self._auto_control.value = self._auto_button.get_active()
        ioctl(VD, v4l2.VIDIOC_S_CTRL, self._auto_control)

        if self._auto_control.value == True:
            self._hscale.set_sensitive(False)
        else:
            self._hscale.set_sensitive(True)


    def __adjustment_changed_cb(self, adj_):
        self._control.value = int(self._adjustment.value)
        ioctl(VD, v4l2.VIDIOC_S_CTRL, self._control)


class XoScopeActivity(activity.Activity):
    """XoScopeActivity class as specified in activity.info"""

    def __init__(self, handle):
        """Set up the XoScope activity."""
        activity.Activity.__init__(self, handle)

        # we do not have collaboration features
        # make the share option insensitive
        self.max_participants = 1

        self._capturing = False

        self._setup_toolbar()

        if OLD_TOOLBAR:
            self.toolbox = ActivityToolbox(self)
            self.set_toolbox(self.toolbox)
            self.toolbox.show()

            activity_toolbar = self.toolbox.get_activity_toolbar()
            activity_toolbar = gtk.Toolbar()
            self.toolbox.add_toolbar(_('Actions'), activity_toolbar)
            self.toolbox.set_current_toolbar(1)
            actions_toolbar = activity_toolbar
        else:
            toolbar_box = ToolbarBox()
            self.activity_button = ActivityToolbarButton(self)
            toolbar_box.toolbar.insert(self.activity_button, 0)
            self.activity_button.show()
            self.set_toolbar_box(toolbar_box)

            toolbar_box.show()


            activity_toolbar = self.activity_button.page
            actions_toolbar = self.get_toolbar_box().toolbar
            actions_toolbar.show()




        if self._check_available_control(v4l2.V4L2_CID_EXPOSURE):
            self._exposure_button = ToolButton('exposure')
            self._exposure_button.set_palette(ScalePalette('Exposure',\
                    v4l2.V4L2_CID_EXPOSURE))
            actions_toolbar.insert(self._exposure_button, -1)
            self._exposure_button.show()

        if self._check_available_control(v4l2.V4L2_CID_BRIGHTNESS):
            self._brightness_button = ToolButton('brightness')
            self._brightness_button.set_palette(ScalePalette('Brightness',\
                    v4l2.V4L2_CID_BRIGHTNESS,
                    self._check_available_control(
                        v4l2.V4L2_CID_AUTOBRIGHTNESS)))
            actions_toolbar.insert(self._brightness_button, -1)
            self._brightness_button.show()

        if self._check_available_control(v4l2.V4L2_CID_CONTRAST):
            self._contrast_button = ToolButton('contrast')
            self._contrast_button.set_palette(ScalePalette('Contrast',\
                    v4l2.V4L2_CID_CONTRAST))
            actions_toolbar.insert(self._contrast_button, -1)
            self._contrast_button.show()

        if self._check_available_control(v4l2.V4L2_CID_SATURATION):
            self._saturation_button = ToolButton('saturation')
            self._saturation_button.set_palette(ScalePalette('Saturation',\
                    v4l2.V4L2_CID_SATURATION))
            actions_toolbar.insert(self._saturation_button, -1)
            self._saturation_button.show()

        if self._check_available_control(v4l2.V4L2_CID_GAIN):
            self._gain_button = ToolButton('gain')
            self._gain_button.set_palette(ScalePalette('Gain',\
                    v4l2.V4L2_CID_GAIN,
                    self._check_available_control(v4l2.V4L2_CID_AUTOGAIN)))
            actions_toolbar.insert(self._gain_button, -1)
            self._gain_button.show()

        if self._check_available_control(
                v4l2.V4L2_CID_WHITE_BALANCE_TEMPERATURE):
            self._white_balance_button = ToolButton('white_balance')
            self._white_balance_button.set_palette(ScalePalette('White'
                    ' balance', v4l2.V4L2_CID_WHITE_BALANCE_TEMPERATURE,
                    self._check_available_control(
                        v4l2.V4L2_CID_AUTO_WHITE_BALANCE)))
            actions_toolbar.insert(self._white_balance_button, -1)
            self._white_balance_button.show()

        if self._check_available_control(v4l2.V4L2_CID_HUE):
            self._color_tone_button = ToolButton('color_tone')
            self._color_tone_button.set_palette(ScalePalette('Color'
                    ' tone', v4l2.V4L2_CID_HUE,
                    self._check_available_control(
                        v4l2.V4L2_CID_HUE_AUTO)))
            actions_toolbar.insert(self._color_tone_button, -1)
            self._color_tone_button.show()

        ## Keep for adding zoom functionality later on.
        #self._zoom_button = ToolButton('zoom')
        #self._zoom_button.set_palette(ScalePalette('Zoom'))
        #actions_toolbar.insert(self._zoom_button, -1)
        #self._zoom_button.show()

        self._photo_button = ToolButton('photo')
        self._photo_button.connect('clicked',
                self.__capture_image_cb)
        actions_toolbar.insert(self._photo_button, -1)
        self._photo_button.show()

        separator = gtk.SeparatorToolItem()
        separator.props.draw = False
        separator.set_expand(True)
        actions_toolbar.insert(separator, -1)
        separator.show()

        if not OLD_TOOLBAR:
            activity_stop = StopButton(self)
            toolbar_box.toolbar.insert(activity_stop, -1)
            activity_stop.show()

        actions_toolbar.show()
        activity_toolbar.show()

        self._main_view = gtk.HBox()

        self._movie_window = gtk.DrawingArea()
        self._main_view.pack_start(self._movie_window)
        self._movie_window.modify_bg(gtk.STATE_NORMAL,
                self._movie_window.style.black)
        self._movie_window.show()

        self._vtray = VTray()
        #self._vtray.props.align = True
        #self._vtray.props.drag_active = True
        #self._main_view.pack_start(self._vtray, False, False)
        self._vtray.set_size_request(120, -1)
        self._vtray.show()

        self.xoscope = gst.Pipeline('xoscope_pipe')
        self.xoscope.set_state(gst.STATE_NULL)
        self.xoscope.delay = 5000000000L
        camsrc = gst.element_factory_make('v4l2src', 'camsrc')
        caps = gst.Caps('video/x-raw-yuv,format=(fourcc)YUY2')
        filt = gst.element_factory_make('capsfilter', 'filter')
        filt.set_property('caps', caps)
        ffmpegcolorspace = gst.element_factory_make('ffmpegcolorspace',
                'ffmpegcolorspace')
        disp_sink = gst.element_factory_make('xvimagesink', 'disp_sink')

        self.image_sink = gst.element_factory_make('gdkpixbufsink',
                'image_sink')
        #self.image_sink.set_property('post_messages', False)
        #self.image_sink.connect('pixbuf', self.__pixbuf_cb)
        ffmpegcolorspace2 = gst.element_factory_make('ffmpegcolorspace',
                'ffmpegcolorspace2')

        tee = gst.element_factory_make('tee', 'tee')
        queue = gst.element_factory_make('queue', 'dispqueue')
        queue.set_property('leaky', True)
        #queue.set_property('silent', True)
        queue.set_property('max-size-buffers', 10)

        self._queue2 = gst.element_factory_make('queue', 'imagequeue')
        self._queue2.set_property('leaky', True)
        #self._queue2.set_property('silent', True)
        self._queue2.set_property('max-size-buffers', 1)

        self.xoscope.add(camsrc, filt, ffmpegcolorspace, disp_sink,
                tee, queue, self._queue2, self.image_sink, ffmpegcolorspace2)
        gst.element_link_many(camsrc, filt, ffmpegcolorspace, tee,
                queue, disp_sink)
        gst.element_link_many(tee, self._queue2, ffmpegcolorspace2,
                self.image_sink)

        bus = self.xoscope.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect('message', self.__on_message_cb)
        bus.connect('sync-message::element', self.__on_sync_message_cb)
        self.xoscope.set_state(gst.STATE_PLAYING)
        self._queue2.set_state(gst.STATE_PAUSED)

        self._main_view.show()
        self.set_canvas(self._main_view)

    def __capture_image_cb(self, w):
        logging.debug('capturing_image_cb')
        if not self._capturing:
            self._queue2.set_state(gst.STATE_PLAYING)
            self._capturing = True
            #self.image_sink.set_property('post_messages', True)

    #def __start_stop_cb(self, w):
    #    if self.button.get_label() == 'Start':
    #        self.button.set_label('Stop')
    #        self.xoscope.set_state(gst.STATE_PLAYING)
    #    else:
    #        self.xoscope.set_state(gst.STATE_NULL)
    #        self.button.set_label('Start')

    def _check_available_control(self, v4l2_control):
        control = v4l2.v4l2_control(v4l2_control)
        try:
            ioctl(VD, v4l2.VIDIOC_QUERYCTRL, control)
        except IOError:
            logging.exception('error setting control')
            return False
        else:
            return v4l2_control

    def __on_message_cb(self, bus, message):
        t = message.type
        #if t == gst.MESSAGE_EOS:
        #    self.xoscope.set_state(gst.STATE_NULL)
        #elif t == gst.MESSAGE_ERROR:
        #    err, debug = message.parse_error()
        #    print 'Error: %s' % err, debug
            #self.xoscope.set_state(gst.STATE_NULL)

    def __on_sync_message_cb(self, bus, message):
        if message.structure is None:
            return
        message_name = message.structure.get_name()
        if message_name == 'prepare-xwindow-id':
            # Assign the viewport
            gtk.gdk.threads_enter()
            imagesink = message.src
            imagesink.set_property('force-aspect-ratio', True)
            imagesink.set_xwindow_id(self._movie_window.window.xid)
            #sink.set_xwindow_id(window.window.xid)
            imagesink.expose()
            gtk.gdk.threads_leave()

        if message_name == "pixbuf":
            if self._capturing:
                self._queue2.set_state(gst.STATE_PAUSED)
                pixbuf = message.structure['pixbuf']
                #pixbuf.save("file_aaaa.jpg", "jpeg", {"quality":"100"})
                self._write_jpeg_image('image_' + str(int(time.time())) +
                        '.jpg', pixbuf)
                self._capturing = False
        #    self.image_sink.set_property('post_messages', False)

    def _write_jpeg_image(self, filename, pixbuf):

        #Write the actual file to the tmp directory of this activity's root.
        file_path = os.path.join(self.get_activity_root(), 'instance', filename)
        #thumbname = 'thumb' + filename
        #thumbImg = generateThumbnail(pixbuf, 100.0/75.0, 100, 75)
        #thumb_path = os.path.join(self.get_activity_root(), 'instance',
        #        thumbname)

        pixbuf.save(file_path, "jpeg", {"quality":"100"})
        #tray_image = gtk.Image()
        #tray_image.set_from_pixbuf(pixbuf)
        #tray_image.set_size_request(100, 75)
        #self._vtray.add_item(tray_image)

        #tray_label = TrayButton()
        #tray_label.set_label('Yippee!!')

        #self._vtray.add_item(tray_label)
        #tray_label.show()

        #thumbImg.write_to_png(thumb_path)
        #thumb_pixbuf = gtk.gdk.pixbuf_new_from_file(thumb_path)
        #thumb_image = gtk.Image()
        #thumb_image.set_from_pixbuf(thumb_pixbuf)

        #thumb_tray_button = TrayButton()
        #thumb_tray_button.set_icon_widget(thumb_image)
        #self._vtray.add_item(thumb_tray_button)
        #thumb_image.show()


        journal_entry = datastore.create()
        journal_entry.metadata['title'] = filename
        journal_entry.metadata['mime_type'] = 'image/jpeg'
        journal_entry.set_file_path(file_path)
        datastore.write(journal_entry)
        journal_entry.destroy()

        ###   # Create a datastore object
        ###   file_dsobject = datastore.create()

        ###   # Write any metadata (here we specifically set the title of the file and
        ###   # specify that this is a plain text file).
        ###   file_dsobject.metadata['title'] = filename
        ###   file_dsobject.metadata['mime_type'] = 'image/jpeg'


        ###   #Set the file_path in the datastore.
        ###   file_dsobject.set_file_path(file_path)

        ###   datastore.write(file_dsobject)
        ###   return file_dsobject

    def _setup_toolbar(self):
        pass
